{ -*- buffer-read-only: t -*- }

{ Unit automatically generated by image-to-pascal tool,
  to embed images in Pascal source code.
  @exclude (Exclude this unit from PasDoc documentation.) }
unit V3DSceneImages;

interface

uses CastleImages;

function Warning_icon: TRGBAlphaImage;

function Examine: TRGBAlphaImage;

function Walk: TRGBAlphaImage;

function Fly: TRGBAlphaImage;

function Open: TRGBAlphaImage;

function Screenshot: TRGBAlphaImage;

function TooltipArrow: TRGBAlphaImage;

function Light_gizmo: TRGBAlphaImage;

function Animations: TRGBAlphaImage;

function Panel: TRGBAlphaImage;

function PanelSeparator: TRGBImage;

function Examine_tooltip: TRGBImage;

function Walk_fly_tooltip: TRGBImage;

implementation

uses SysUtils;

{ Actual image data is included from another file, with a deliberately
  non-Pascal file extension ".image_data". This way online code analysis
  tools will NOT consider this source code as an uncommented Pascal code
  (which would be unfair --- the image data file is autogenerated
  and never supposed to be processed by a human). }
{$I v3dsceneimages.image_data}

initialization
finalization
  FreeAndNil(FWarning_icon);
  FreeAndNil(FExamine);
  FreeAndNil(FWalk);
  FreeAndNil(FFly);
  FreeAndNil(FOpen);
  FreeAndNil(FScreenshot);
  FreeAndNil(FTooltipArrow);
  FreeAndNil(FLight_gizmo);
  FreeAndNil(FAnimations);
  FreeAndNil(FPanel);
  FreeAndNil(FPanelSeparator);
  FreeAndNil(FExamine_tooltip);
  FreeAndNil(FWalk_fly_tooltip);
end.