{ view3dscene and tovrmlx3d version and URL constants. }
unit V3DSceneVersion;
interface
const
  Version = '4.2.0';
  View3dsceneURL = 'https://castle-engine.io/view3dscene.php';
  SupportURL = 'https://castle-engine.io/donate.php';
implementation
end.
