{ -*- buffer-read-only: t -*- }
{ DON'T EDIT -- this file was automatically generated from "main.castle-user-interface" }
'{' + LineEnding +
'  "FullSize" : true,' + LineEnding +
'  "Name" : "Group1",' + LineEnding +
'  "$Children" : [' + LineEnding +
'    {' + LineEnding +
'      "Height" : 5.2000000000000000E+001,' + LineEnding +
'      "Name" : "ToolbarPanel",' + LineEnding +
'      "VerticalAnchorParent" : "vpTop",' + LineEnding +
'      "VerticalAnchorSelf" : "vpTop",' + LineEnding +
'      "WidthFraction" : 1.0000000000000000E+000,' + LineEnding +
'      "$Children" : [' + LineEnding +
'        {' + LineEnding +
'          "Content" : {' + LineEnding +
'            "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'          },' + LineEnding +
'          "FullSize" : true,' + LineEnding +
'          "Height" : 5.2000000000000000E+001,' + LineEnding +
'          "Name" : "ToolbarBackground",' + LineEnding +
'          "Stretch" : true,' + LineEnding +
'          "VerticalAnchorParent" : "vpTop",' + LineEnding +
'          "VerticalAnchorSelf" : "vpTop",' + LineEnding +
'          "WidthFraction" : 1.0000000000000000E+000,' + LineEnding +
'          "$Children" : [' + LineEnding +
'            {' + LineEnding +
'              "Name" : "ToolbarHorizGroup",' + LineEnding +
'              "Padding" : 5.0000000000000000E+000,' + LineEnding +
'              "Spacing" : 8.0000000000000000E+000,' + LineEnding +
'              "VerticalAnchorParent" : "vpTop",' + LineEnding +
'              "VerticalAnchorSelf" : "vpTop",' + LineEnding +
'              "$Children" : [' + LineEnding +
'                {' + LineEnding +
'                  "Caption" : "Open",' + LineEnding +
'                  "CustomBackgroundDisabled" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundFocused" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundNormal" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundPressed" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "HorizontalAnchorDelta" : 5.0000000000000000E+000,' + LineEnding +
'                  "Image" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "Name" : "ButtonOpen",' + LineEnding +
'                  "VerticalAnchorParent" : "vpMiddle",' + LineEnding +
'                  "VerticalAnchorSelf" : "vpMiddle",' + LineEnding +
'                  "$Children" : [' + LineEnding +
'                  ],' + LineEnding +
'                  "$ClassName" : "TCastleButton"' + LineEnding +
'                },' + LineEnding +
'                {' + LineEnding +
'                  "Content" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "Height" : 4.2000000000000000E+001,' + LineEnding +
'                  "HorizontalAnchorDelta" : 9.7000000000000000E+001,' + LineEnding +
'                  "Name" : "PanelSeparator1",' + LineEnding +
'                  "Stretch" : true,' + LineEnding +
'                  "VerticalAnchorParent" : "vpMiddle",' + LineEnding +
'                  "VerticalAnchorSelf" : "vpMiddle",' + LineEnding +
'                  "Width" : 2.0000000000000000E+000,' + LineEnding +
'                  "$Children" : [' + LineEnding +
'                  ],' + LineEnding +
'                  "$ClassName" : "TCastleImageControl"' + LineEnding +
'                },' + LineEnding +
'                {' + LineEnding +
'                  "Content" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "Height" : 4.2000000000000000E+001,' + LineEnding +
'                  "HorizontalAnchorDelta" : 1.0700000000000000E+002,' + LineEnding +
'                  "Name" : "PanelSeparator2",' + LineEnding +
'                  "Stretch" : true,' + LineEnding +
'                  "VerticalAnchorParent" : "vpMiddle",' + LineEnding +
'                  "VerticalAnchorSelf" : "vpMiddle",' + LineEnding +
'                  "Width" : 2.0000000000000000E+000,' + LineEnding +
'                  "$Children" : [' + LineEnding +
'                  ],' + LineEnding +
'                  "$ClassName" : "TCastleImageControl"' + LineEnding +
'                },' + LineEnding +
'                {' + LineEnding +
'                  "Caption" : "Collisions",' + LineEnding +
'                  "CustomBackgroundDisabled" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundFocused" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundNormal" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundPressed" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "HorizontalAnchorDelta" : 1.1700000000000000E+002,' + LineEnding +
'                  "Image" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "MinImageHeight" : 2.2000000000000000E+001,' + LineEnding +
'                  "Name" : "ButtonCollisions",' + LineEnding +
'                  "Toggle" : true,' + LineEnding +
'                  "VerticalAnchorParent" : "vpMiddle",' + LineEnding +
'                  "VerticalAnchorSelf" : "vpMiddle",' + LineEnding +
'                  "$Children" : [' + LineEnding +
'                  ],' + LineEnding +
'                  "$ClassName" : "TCastleButton"' + LineEnding +
'                },' + LineEnding +
'                {' + LineEnding +
'                  "Caption" : "Screenshot",' + LineEnding +
'                  "CustomBackgroundDisabled" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundFocused" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundNormal" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundPressed" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "HorizontalAnchorDelta" : 2.5000000000000000E+002,' + LineEnding +
'                  "Image" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "MinImageHeight" : 2.2000000000000000E+001,' + LineEnding +
'                  "Name" : "ButtonScreenshot",' + LineEnding +
'                  "VerticalAnchorParent" : "vpMiddle",' + LineEnding +
'                  "VerticalAnchorSelf" : "vpMiddle",' + LineEnding +
'                  "$Children" : [' + LineEnding +
'                  ],' + LineEnding +
'                  "$ClassName" : "TCastleButton"' + LineEnding +
'                },' + LineEnding +
'                {' + LineEnding +
'                  "Caption" : "Animations",' + LineEnding +
'                  "CustomBackgroundDisabled" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundFocused" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundNormal" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "CustomBackgroundPressed" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "HorizontalAnchorDelta" : 4.0000000000000000E+002,' + LineEnding +
'                  "Image" : {' + LineEnding +
'                    "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'                  },' + LineEnding +
'                  "MinImageHeight" : 2.2000000000000000E+001,' + LineEnding +
'                  "Name" : "ButtonAnimations",' + LineEnding +
'                  "VerticalAnchorParent" : "vpMiddle",' + LineEnding +
'                  "VerticalAnchorSelf" : "vpMiddle",' + LineEnding +
'                  "$Children" : [' + LineEnding +
'                  ],' + LineEnding +
'                  "$ClassName" : "TCastleButton"' + LineEnding +
'                }' + LineEnding +
'              ],' + LineEnding +
'              "$ClassName" : "TCastleHorizontalGroup"' + LineEnding +
'            }' + LineEnding +
'          ],' + LineEnding +
'          "$ClassName" : "TCastleImageControl"' + LineEnding +
'        }' + LineEnding +
'      ],' + LineEnding +
'      "$ClassName" : "TCastleUserInterface"' + LineEnding +
'    },' + LineEnding +
'    {' + LineEnding +
'      "Caption" : "Warnings",' + LineEnding +
'      "CustomBackgroundDisabled" : {' + LineEnding +
'        "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'      },' + LineEnding +
'      "CustomBackgroundFocused" : {' + LineEnding +
'        "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'      },' + LineEnding +
'      "CustomBackgroundNormal" : {' + LineEnding +
'        "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'      },' + LineEnding +
'      "CustomBackgroundPressed" : {' + LineEnding +
'        "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'      },' + LineEnding +
'      "HorizontalAnchorDelta" : -5.0000000000000000E+000,' + LineEnding +
'      "HorizontalAnchorParent" : "hpRight",' + LineEnding +
'      "HorizontalAnchorSelf" : "hpRight",' + LineEnding +
'      "Image" : {' + LineEnding +
'        "$ClassName" : "TCastleImagePersistent"' + LineEnding +
'      },' + LineEnding +
'      "MinImageHeight" : 2.2000000000000000E+001,' + LineEnding +
'      "Name" : "ButtonWarnings",' + LineEnding +
'      "VerticalAnchorDelta" : -5.0000000000000000E+000,' + LineEnding +
'      "VerticalAnchorParent" : "vpTop",' + LineEnding +
'      "VerticalAnchorSelf" : "vpTop",' + LineEnding +
'      "$Children" : [' + LineEnding +
'      ],' + LineEnding +
'      "$ClassName" : "TCastleButton"' + LineEnding +
'    }' + LineEnding +
'  ],' + LineEnding +
'  "$ClassName" : "TCastleUserInterface"' + LineEnding +
'}' + LineEnding +
''
